@@ -263,7 +263,6 @@ DatabaseSession::DataInsert(CSSM_DB_HANDLE inDbHandle,
                             const CssmData *inData,
                             CSSM_DB_UNIQUE_RECORD_PTR &outUniqueId)
 {
-	debug("dbsession", "%p DataInsert(%lx,%lx)", this, inDbHandle, inRecordType);
     DbContext &aDbContext = findDbContext(inDbHandle);
     outUniqueId = aDbContext.mDatabase.dataInsert(aDbContext, inRecordType, inAttributes, inData);
 }
@@ -273,7 +272,6 @@ void
 DatabaseSession::DataDelete(CSSM_DB_HANDLE inDbHandle,
                             const CSSM_DB_UNIQUE_RECORD &inUniqueRecordIdentifier)
 {
-	debug("dbsession", "%p DataDelete(%lx)", this, inDbHandle);
     DbContext &aDbContext = findDbContext(inDbHandle);
     aDbContext.mDatabase.dataDelete(aDbContext, inUniqueRecordIdentifier);
 }
@@ -287,7 +285,6 @@ DatabaseSession::DataModify(CSSM_DB_HANDLE inDbHandle,
                             const CssmData *inDataToBeModified,
                             CSSM_DB_MODIFY_MODE inModifyMode)
 {
-	debug("dbsession", "%p DataModify(%lx,%lx)", this, inDbHandle, inRecordType);
     DbContext &aDbContext = findDbContext(inDbHandle);
     aDbContext.mDatabase.dataModify(aDbContext, inRecordType, inoutUniqueRecordIdentifier,
                                      inAttributesToBeModified, inDataToBeModified, inModifyMode);
@@ -300,7 +297,6 @@ DatabaseSession::DataGetFirst(CSSM_DB_HANDLE inDbHandle,
                               CssmData *inoutData,
                               CSSM_DB_UNIQUE_RECORD_PTR &outUniqueId)
 {
-	debug("dbsession", "%p DataGetFirst(%lx)", this, inDbHandle);
     DbContext &aDbContext = findDbContext(inDbHandle);
 	
 	return aDbContext.mDatabase.dataGetFirst(aDbContext, inQuery,
@@ -314,7 +310,6 @@ DatabaseSession::DataGetNext(CSSM_DB_HANDLE inDbHandle,
                              CssmData *inoutData,
                              CSSM_DB_UNIQUE_RECORD_PTR &outUniqueRecord)
 {
-	debug("dbsession", "%p DataGetNext(%lx)", this, inDbHandle);
     DbContext &aDbContext = findDbContext(inDbHandle);
 
 	return aDbContext.mDatabase.dataGetNext(aDbContext, inResultsHandle, inoutAttributes,
@@ -325,7 +320,6 @@ void
 DatabaseSession::DataAbortQuery(CSSM_DB_HANDLE inDbHandle,
                                 CSSM_HANDLE inResultsHandle)
 {
-	debug("dbsession", "%p DataAbortQuery(%lx)", this, inDbHandle);
     DbContext &aDbContext = findDbContext(inDbHandle);
     aDbContext.mDatabase.dataAbortQuery(aDbContext, inResultsHandle);
 }
@@ -336,7 +330,6 @@ DatabaseSession::DataGetFromUniqueRecordId(CSSM_DB_HANDLE inDbHandle,
                                            CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR inoutAttributes,
                                            CssmData *inoutData)
 {
-	debug("dbsession", "%p DataGetFromUniqueId(%lx)", this, inDbHandle);
     DbContext &aDbContext = findDbContext(inDbHandle);
     aDbContext.mDatabase.dataGetFromUniqueRecordId(aDbContext, inUniqueRecord,
                                                    inoutAttributes, inoutData);
